local padding = 5
Hooks:OverrideFunction(ContractBrokerHeistItem, "init", function(self, parent_panel, job_data, idx, ...)

    self._parent = parent_panel
	self._job_data = job_data
	local job_tweak = tweak_data.narrative:job_data(job_data.job_id)
	local contact = job_tweak.contact
	local contact_tweak = tweak_data.narrative.contacts[contact]

    local compact_mode = contractorbrokerimp.options.compact_mode

    local heat_text, heat_color

    if contractorbrokerimp.options.plusorminus then
        heat_text, heat_color = self:get_job_heat_icon(self._job_data.job_id)
    else
        heat_text, heat_color = self:get_job_heat_text(self._job_data.job_id)
    end

    if compact_mode then
        self._panel = parent_panel:panel({
            halign = "grow",
            layer = 10,
            h = 20,
            x = 0,
            valign = "top",
            y = 5 + 20 * (idx - 1)
        })
        self._background = self._panel:rect({
            blend_mode = "add",
            alpha = 0.4,
            halign = "grow",
            layer = -1,
            valign = "grow",
            h = self._panel:h(),
            color = job_data.enabled and tweak_data.screen_colors.button_stage_3 or tweak_data.screen_colors.important_1
        })

        self._background:set_visible(false)

        local img_size = self._panel:h() - padding
        self._image_panel = self._panel:panel({
            halign = "left",
            layer = 1,
            x = 0,
            valign = "top",
            y = padding,
            visible = false,
        })
        local has_image = false

        local job_name = self._panel:text({
            layer = 1,
            vertical = "top",
            align = "left",
            halign = "left",
            valign = "top",
            x = 5,
            text = managers.localization:to_upper_text(job_tweak.name_id),
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.8,
            color = job_data.enabled and tweak_data.screen_colors.button_stage_3 or tweak_data.screen_colors.important_1
        })
        self:make_fine_text(job_name)
        job_name:set_center_y(self._panel:h() / 2)

        local contact_name = self._panel:text({
            alpha = 0,
            vertical = "top",
            layer = 1,
            align = "left",
            halign = "left",
            valign = "top",
            text = "",
            font = tweak_data.menu.pd2_large_font,
            font_size = 0,
            color = tweak_data.screen_colors.text
        })

        self:make_fine_text(contact_name)
        local dlc_name, dlc_color = self:get_dlc_name_and_color(job_tweak)
        local dlc_name = self._panel:text({
            alpha = 1,
            vertical = "top",
            layer = 1,
            align = "left",
            halign = "left",
            valign = "top",
            text = dlc_name,
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.7,
            color = dlc_color
        })

        self:make_fine_text(dlc_name)
        dlc_name:set_left(job_name:right() + 5)
        dlc_name:set_center_y(job_name:center_y())

        if job_data.is_new then
            local new_name = self._panel:text({
                alpha = 1,
                vertical = "top",
                layer = 1,
                align = "left",
                halign = "left",
                valign = "top",
                text = managers.localization:to_upper_text("menu_new"),
                font = tweak_data.menu.pd2_medium_font,
                font_size = tweak_data.menu.pd2_medium_font_size * 0.7,
                color = Color(255, 105, 254, 59) / 255
            })

            self:make_fine_text(new_name)
            new_name:set_left((dlc_name:text() ~= "" and dlc_name or job_name):right() + 5)
            new_name:set_center_y(job_name:center_y())
        end

        local icons_panel = self._panel:panel({
            valign = "top",
            halign = "right",
            h = job_name:h(),
            w = self._panel:w() * 0.5
        })

        icons_panel:set_right(self._panel:right() - padding)
        icons_panel:set_top(job_name:top())

        local icon_size = icons_panel:h()
        local last_icon = nil
        self._favourite = icons_panel:bitmap({
            texture = "guis/dlcs/bro/textures/pd2/favourite",
            halign = "right",
            alpha = 0.8,
            valign = "top",
            color = Color.white,
            w = icon_size,
            h = icon_size
        })

        self._favourite:set_right(icons_panel:w())

        last_icon = self._favourite
        local day_text = icons_panel:text({
            layer = 1,
            vertical = "center",
            align = "right",
            halign = "right",
            valign = "top",
            text = self:get_heist_day_text(),
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.7,
            color = tweak_data.screen_colors.text
        })

        self:make_fine_text(day_text)
        day_text:set_right(last_icon:left() - 5)
        day_text:set_center_y(job_name:center_y())

        last_icon = day_text
        local length_icon = icons_panel:text({
            layer = 1,
            vertical = "center",
            align = "right",
            halign = "right",
            valign = "top",
            text = self:get_heist_day_icon(),
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.7,
            color = tweak_data.screen_colors.text
        })

        self:make_fine_text(length_icon)
        length_icon:set_right(last_icon:left() - padding)
        length_icon:set_center_y(job_name:center_y())

        last_icon = length_icon

        if self:is_stealthable() then
            local stealth = icons_panel:text({
                layer = 1,
                vertical = "center",
                align = "middle",
                halign = "right",
                valign = "top",
                text = managers.localization:get_default_macro("BTN_GHOST"),
                font = tweak_data.menu.pd2_medium_font,
                font_size = tweak_data.menu.pd2_medium_font_size * 0.8,
                color = tweak_data.screen_colors.text
            })

            self:make_fine_text(stealth)
            stealth:set_right(last_icon:left() - padding)
            last_icon = stealth
        end

        local last_played = icons_panel:text({
            alpha = 0.7,
            vertical = "center",
            layer = 1,
            align = "left",
            halign = "right",
            valign = "top",
            text = self:get_last_played_text(),
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.8,
            color = tweak_data.screen_colors.text
        })

        self:make_fine_text(last_played)
        last_played:set_right(last_icon:left() - padding)
        last_played:set_center_y(job_name:center_y())

        local heat = self._panel:text({
            alpha = 1,
            vertical = "top",
            layer = 1,
            align = "right",
            halign = "right",
            valign = "top",
            text = heat_text,
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.7,
            color = heat_color
        })
        self:make_fine_text(heat)
        heat:set_right(dlc_name:right() + 35)
    else
        self._panel = parent_panel:panel({
            halign = "grow",
            layer = 10,
            h = 90,
            x = 0,
            valign = "top",
            y = 90 * (idx - 1)
        })
        self._background = self._panel:rect({
            blend_mode = "add",
            alpha = 0.4,
            halign = "grow",
            layer = -1,
            valign = "grow",
            h = self._panel:h() - padding,
            color = job_data.enabled and tweak_data.screen_colors.button_stage_3 or tweak_data.screen_colors.important_1
        })
        self._background:set_visible(false)

        local img_size = self._panel:h() - padding
        self._image_panel = self._panel:panel({
            halign = "left",
            layer = 1,
            x = 0,
            valign = "top",
            y = padding,
            w = img_size * 1.7777777777777777,
            h = img_size
        })

        local has_image = false
        if job_tweak.contract_visuals and job_tweak.contract_visuals.preview_image then
            local data = job_tweak.contract_visuals.preview_image
            local path, rect = nil

            if data.id then
                path = "guis/dlcs/" .. (data.folder or "bro") .. "/textures/pd2/crimenet/" .. data.id
                rect = data.rect
            elseif data.icon then
                path, rect = tweak_data.hud_icons:get_icon_data(data.icon)
            end

            if path and DB:has(Idstring("texture"), path) then
                self._image_panel:bitmap({
                    valign = "scale",
                    layer = 2,
                    blend_mode = "add",
                    halign = "scale",
                    texture = path,
                    texture_rect = rect,
                    w = self._image_panel:w(),
                    h = self._image_panel:h(),
                    color = Color.white
                })

                self._image = self._image_panel:rect({
                    alpha = 1,
                    layer = 1,
                    color = Color.black
                })
                has_image = true
            end
        end

        if not has_image then
            local color = Color.red
            local error_message = "Missing Preview Image"

            self._image_panel:rect({
                alpha = 0.4,
                layer = 1,
                color = color
            })
            self._image_panel:text({
                vertical = "center",
                wrap = true,
                align = "center",
                word_wrap = true,
                layer = 2,
                text = error_message,
                font = tweak_data.menu.pd2_small_font,
                font_size = tweak_data.menu.pd2_small_font_size
            })
            BoxGuiObject:new(self._image_panel:panel({
                layer = 100
            }), {
                sides = {
                    1,
                    1,
                    1,
                    1
                }
            })
        end

        local job_name = self._panel:text({
            layer = 1,
            vertical = "top",
            align = "left",
            halign = "left",
            valign = "top",
            text = managers.localization:to_upper_text(job_tweak.name_id),
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size,
            color = job_data.enabled and tweak_data.screen_colors.text or tweak_data.screen_colors.important_1
        })

        self:make_fine_text(job_name)
        job_name:set_left(self._image_panel:right() + padding * 2)
        job_name:set_top(self._panel:h() * 0.5 + padding * 0.5)

        local contact_name = self._panel:text({
            alpha = 0.8,
            vertical = "top",
            layer = 1,
            align = "left",
            halign = "left",
            valign = "top",
            text = managers.localization:to_upper_text(contact_tweak.name_id),
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.9,
            color = tweak_data.screen_colors.text
        })

        self:make_fine_text(contact_name)
        contact_name:set_left(job_name:left())
        contact_name:set_bottom(job_name:top())

        local dlc_name, dlc_color = self:get_dlc_name_and_color(job_tweak)
        local dlc_name = self._panel:text({
            alpha = 1,
            vertical = "top",
            layer = 1,
            align = "left",
            halign = "left",
            valign = "top",
            text = dlc_name,
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.9,
            color = dlc_color
        })

        self:make_fine_text(dlc_name)
        dlc_name:set_left(contact_name:right() + 5)
        dlc_name:set_bottom(job_name:top())

        if job_data.is_new then
            local new_name = self._panel:text({
                alpha = 1,
                vertical = "top",
                layer = 1,
                align = "left",
                halign = "left",
                valign = "top",
                text = managers.localization:to_upper_text("menu_new"),
                font = tweak_data.menu.pd2_medium_font,
                font_size = tweak_data.menu.pd2_medium_font_size * 0.9,
                color = Color(255, 105, 254, 59) / 255
            })

            self:make_fine_text(new_name)
            new_name:set_left((dlc_name:text() ~= "" and dlc_name or contact_name):right() + 5)
            new_name:set_bottom(job_name:top())
        end

        local last_played = self._panel:text({
            alpha = 0.7,
            vertical = "top",
            layer = 1,
            align = "right",
            halign = "right",
            valign = "top",
            text = self:get_last_played_text(),
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.8,
            color = tweak_data.screen_colors.text
        })

        self:make_fine_text(last_played)
        last_played:set_right(self._panel:right() - padding)
        last_played:set_bottom(job_name:top())

        local icons_panel = self._panel:panel({
            valign = "top",
            halign = "right",
            h = job_name:h(),
            w = self._panel:w() * 0.3
        })

        icons_panel:set_right(self._panel:right() - padding)
        icons_panel:set_top(job_name:top())

        local icon_size = icons_panel:h()
        local last_icon = nil
        self._favourite = icons_panel:bitmap({
            texture = "guis/dlcs/bro/textures/pd2/favourite",
            halign = "right",
            alpha = 0.8,
            valign = "top",
            color = Color.white,
            w = icon_size,
            h = icon_size
        })

        self._favourite:set_right(icons_panel:w())

        last_icon = self._favourite
        local day_text = icons_panel:text({
            layer = 1,
            vertical = "bottom",
            align = "right",
            halign = "right",
            valign = "top",
            text = self:get_heist_day_text(),
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.9,
            color = tweak_data.screen_colors.text
        })

        self:make_fine_text(day_text)
        day_text:set_right(last_icon:left() - 5)
        day_text:set_bottom(icons_panel:h())

        last_icon = day_text
        local length_icon = icons_panel:text({
            layer = 1,
            vertical = "bottom",
            align = "right",
            halign = "right",
            valign = "top",
            text = self:get_heist_day_icon(),
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.8,
            color = tweak_data.screen_colors.text
        })

        self:make_fine_text(length_icon)
        length_icon:set_right(last_icon:left() - padding)
        length_icon:set_top(2)

        last_icon = length_icon

        if self:is_stealthable() then
            local stealth = icons_panel:text({
                layer = 1,
                vertical = "top",
                align = "right",
                halign = "right",
                valign = "top",
                text = managers.localization:get_default_macro("BTN_GHOST"),
                font = tweak_data.menu.pd2_medium_font,
                font_size = tweak_data.menu.pd2_medium_font_size,
                color = tweak_data.screen_colors.text
            })

            self:make_fine_text(stealth)
            stealth:set_right(last_icon:left() - padding)

            last_icon = stealth
        end

        local heat = self._panel:text({
            alpha = 1,
            vertical = "top",
            layer = 1,
            align = "right",
            halign = "right",
            valign = "top",
            text = heat_text,
            font = tweak_data.menu.pd2_medium_font,
            font_size = tweak_data.menu.pd2_medium_font_size * 0.7,
            color = heat_color
        })
        self:make_fine_text(heat)
        heat:set_right(self._panel:right() - 10)
        heat:set_top(10)
    end

    self:refresh()
end)

function ContractBrokerHeistItem:make_fine_text(text)
	local x, y, w, h = text:text_rect()

	text:set_size(w, h)
	text:set_position(math.round(text:x()), math.round(text:y()))
end

function ContractBrokerHeistItem:get_job_heat_text(job_id)
	local heat_text       = ""
	local heat_color      = Color(1,0,1)
	local exp_multiplier  = managers.job:heat_to_experience_multiplier(managers.job:get_job_heat(job_id) or 0)
	local exp_percent     = ((1 - exp_multiplier)*-1)*100

	if exp_percent ~= 0 then
		local prefix  = exp_percent > 0 and "+" or ""
		heat_text = "("..prefix..exp_percent.."%)"
        heat_color = exp_percent > 0 and tweak_data.screen_colors.heat_warm_color or tweak_data.screen_colors.heat_cold_color
    end

	return heat_text, heat_color
end

function ContractBrokerHeistItem:get_job_heat_icon(job_id)
	local heat_text = ""
	local heat_color = Color(1,0,1)
	local multiplier = managers.job:get_job_heat(job_id) or 0
	local exp_multi  = managers.job:heat_to_experience_multiplier(multiplier)*20-20

    if exp_multi ~= 0 then
        heat_text = (exp_multi>0 and ("+"):rep(math.ceil(exp_multi)) or ("-"):rep(-math.floor(exp_multi)))
        heat_color = (exp_multi > 0 and Color.yellow) or Color('E55858')
    end

	return heat_text, heat_color
end